// File vpdef.h for CC1B

/*
	This library requires the following constants to be defined before including the library
	
	#define  _VPDEF_H_VPSLOTS  1  //set number of VP slots (1 to 8)
	#define  _VPDEF_H_vpData0  0x30  //set bank of first vp ram bank
*/

#ifndef _VPDEF_H_
#define _VPDEF_H_
//include other libraries here
#include <memory.h>
#pragma library 1

#ifndef _VPDEF_H_VPSLOTS
#error _VPDEF_H_VPSLOTS must be defined before including <vpdef.h>
#else
#if (_VPDEF_H_VPSLOTS < 1) || (_VPDEF_H_VPSLOTS > 8)
#error _VPDEF_H_VPSLOTS must be in the range 1..8
#endif
#endif

#ifndef _VPDEF_H_vpData0
#error _VPDEF_H_vpData0 must be defined before including <vpdef.h>
#else
#if (_VPDEF_H_vpData0 < 0x10) || ((_VPDEF_H_vpData0 & 0x0F) != 0)
#error _VPDEF_H_vpData0 must start at rambank begin
#endif
#endif

#define VP_TIMERS 0
#define VP_UARTRX 1
#define VP_UARTTX 2

struct vpUartRx {
  char mtUART0rx_count; //number of bits
  char mtUART0rx_divide; //timing counter
  char mtUART0rx_low; //shift register
  char mtUART0rx_high; //value
  //char mtUART0rx_format; //
  bit mtUART0rx_bits_0;//  @mtUART0rx_format.0; //bits 1 and 0 specify number of total bits
  bit mtUART0rx_bits_1;// @mtUART0rx_format.1; //01 -> 9bits, 10 -> 10bits, 11 -> 11bits (start+data+parity+stop)
  bit mtUART0rx_7bits;// @mtUART0rx_format.2; //set for 7 databits (0 = 8 databits)
  bit mtUART0rx_even;// @mtUART0rx_format.3; //set for even parity
  bit mtUART0rx_odd;// @mtUART0rx_format.4; //set for odd parity
  bit mtUART0rx_data;// @mtUART0rx_format.5; //set if valid value, cleared after value grabbed
  bit mtUART0rx_enable;// @mtUART0rx_format.6; //set to enable uart
  bit mtUART0rx_parity;// @mtUART0rx_format.7; //parity bit placeholder
  char mtUART0rx_baud; //baudrate
  char mtUART0rx_datapin; //high nib = port (5 to 9), low nib = pin (0 to 7)
  //bit mtUART0rx_error @mtUART0rx_datapin.3; //set if parity error
  char mtUART0rx_hspin; //high nib = port (5 to 9), low nib = pin (0 to 7)
  //bit mtUART0rx_full @mtUART0rx_hspin.3; //set if buffer full
  char mtUART0rx_headtail; //
  char mtUART0rx_buf[7];
};

struct vpUartTx {
  char mtUART0tx_count; //number of bits
  char mtUART0tx_divide; //timing counter
  char mtUART0tx_low; //shift register
  char mtUART0tx_high; //value
  //char mtUART0tx_format; //
  bit mtUART0tx_bits_0;// @mtUART0tx_format.0; //bits 1 and 0 specify number of total bits
  bit mtUART0tx_bits_1;// @mtUART0tx_format.1; //01 -> 9bits, 10 -> 10bits, 11 -> 11bits (start+data+parity+stop)
  bit mtUART0tx_7bits;// @mtUART0tx_format.2; //set for 7 databits (0 = 8 databits)
  bit mtUART0tx_even;// @mtUART0tx_format.3; //set for even parity
  bit mtUART0tx_odd;// @mtUART0tx_format.4; //set for odd parity
  bit mtUART0tx_data;// @mtUART0tx_format.5; //set if valid value, cleared after value grabbed
  bit mtUART0tx_enable;// @mtUART0tx_format.6; //set to enable uart
  bit mtUART0tx_parity;// @mtUART0tx_format.7; //parity bit placeholder
  char mtUART0tx_baud; //baudrate
  char mtUART0tx_datapin; //high nib = port (5 to 9), low nib = pin (0 to 7)
  //bit mtUART0tx_error @mtUART0tx_datapin.3; //set if parity error
  char mtUART0tx_hspin; //high nib = port (5 to 9), low nib = pin (0 to 7)
  //bit mtUART0tx_full @mtUART0tx_hspin.3; //set if buffer full
  char mtUART0tx_headtail;
  char mtUART0tx_buf[7];
};

//each virtual peripheral has 16 ram bytes that occupy one full rambank
union vpBank {
  char vpRam[16];
  struct vpUartRx rx;
  struct vpUartTx tx;
};

struct vpDef {
  char vpType;
  char vpParameters[16];
};

#if defined _CHIP_SX18_ || defined _CHIP_SX20_ || defined _CHIP_SX28_

#if _VPDEF_H_VPSLOTS == 1
union vpBank vpData0 @_VPDEF_H_vpData0;
#elif _VPDEF_H_VPSLOTS == 2
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x20;
#elif _VPDEF_H_VPSLOTS == 3
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x20;
union vpBank vpData2 @_VPDEF_H_vpData0+0x40;
#elif _VPDEF_H_VPSLOTS == 4
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x20;
union vpBank vpData2 @_VPDEF_H_vpData0+0x40;
union vpBank vpData3 @_VPDEF_H_vpData0+0x60;
#elif _VPDEF_H_VPSLOTS == 5
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x20;
union vpBank vpData2 @_VPDEF_H_vpData0+0x40;
union vpBank vpData3 @_VPDEF_H_vpData0+0x60;
union vpBank vpData4 @_VPDEF_H_vpData0+0x80;
#elif _VPDEF_H_VPSLOTS == 6
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x20;
union vpBank vpData2 @_VPDEF_H_vpData0+0x40;
union vpBank vpData3 @_VPDEF_H_vpData0+0x60;
union vpBank vpData4 @_VPDEF_H_vpData0+0x80;
union vpBank vpData5 @_VPDEF_H_vpData0+0xA0;
#elif _VPDEF_H_VPSLOTS == 7
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x20;
union vpBank vpData2 @_VPDEF_H_vpData0+0x40;
union vpBank vpData3 @_VPDEF_H_vpData0+0x60;
union vpBank vpData4 @_VPDEF_H_vpData0+0x80;
union vpBank vpData5 @_VPDEF_H_vpData0+0xA0;
union vpBank vpData6 @_VPDEF_H_vpData0+0xC0;
#elif _VPDEF_H_VPSLOTS == 8
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x20;
union vpBank vpData2 @_VPDEF_H_vpData0+0x40;
union vpBank vpData3 @_VPDEF_H_vpData0+0x60;
union vpBank vpData4 @_VPDEF_H_vpData0+0x80;
union vpBank vpData5 @_VPDEF_H_vpData0+0xA0;
union vpBank vpData6 @_VPDEF_H_vpData0+0xC0;
union vpBank vpData7 @_VPDEF_H_vpData0+0xE0;
#else
#error Number of _VPDEF_H_VPSLOTS too high
#endif

#endif

#if defined _CHIP_SX48_ || defined _CHIP_SX52_

#if _VPDEF_H_VPSLOTS == 1
union vpBank vpData0 @_VPDEF_H_vpData0;
#elif _VPDEF_H_VPSLOTS == 2
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x10;
#elif _VPDEF_H_VPSLOTS == 3
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x10;
union vpBank vpData2 @_VPDEF_H_vpData0+0x20;
#elif _VPDEF_H_VPSLOTS == 4
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x10;
union vpBank vpData2 @_VPDEF_H_vpData0+0x20;
union vpBank vpData3 @_VPDEF_H_vpData0+0x30;
#elif _VPDEF_H_VPSLOTS == 5
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x10;
union vpBank vpData2 @_VPDEF_H_vpData0+0x20;
union vpBank vpData3 @_VPDEF_H_vpData0+0x30;
union vpBank vpData4 @_VPDEF_H_vpData0+0x40;
#elif _VPDEF_H_VPSLOTS == 6
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x10;
union vpBank vpData2 @_VPDEF_H_vpData0+0x20;
union vpBank vpData3 @_VPDEF_H_vpData0+0x30;
union vpBank vpData4 @_VPDEF_H_vpData0+0x40;
union vpBank vpData5 @_VPDEF_H_vpData0+0x50;
#elif _VPDEF_H_VPSLOTS == 7
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x10;
union vpBank vpData2 @_VPDEF_H_vpData0+0x20;
union vpBank vpData3 @_VPDEF_H_vpData0+0x30;
union vpBank vpData4 @_VPDEF_H_vpData0+0x40;
union vpBank vpData5 @_VPDEF_H_vpData0+0x50;
union vpBank vpData6 @_VPDEF_H_vpData0+0x60;
#elif _VPDEF_H_VPSLOTS == 8
union vpBank vpData0 @_VPDEF_H_vpData0;
union vpBank vpData1 @_VPDEF_H_vpData0+0x10;
union vpBank vpData2 @_VPDEF_H_vpData0+0x20;
union vpBank vpData3 @_VPDEF_H_vpData0+0x30;
union vpBank vpData4 @_VPDEF_H_vpData0+0x40;
union vpBank vpData5 @_VPDEF_H_vpData0+0x50;
union vpBank vpData6 @_VPDEF_H_vpData0+0x60;
union vpBank vpData7 @_VPDEF_H_vpData0+0x70;
#else
#error Number of _VPDEF_H_VPSLOTS too high
#endif

#endif

shadowDef char vpData @vpData0;

#pragma library 0
#endif
// End of file vpdef.h
