// File vplib.h for CC1B

/*
	This library requires the following constants to be defined before including the library
	
	#define  _VPLIB_H_vpList  0x10  //set address of char vpList[_VPDEF_H_VPSLOTS]
	#define  _VPLIB_H_vpFree  0x16  //set address of char vpFree
*/

#ifndef _VPLIB_H_
#define _VPLIB_H_
//include other libraries here
#include <memory.h>
#include <math16.h>
#pragma library 1

#ifndef _VPLIB_H_vpList
#error _VPLIB_H_vpList must be defined before including <vplib.h>
#endif

#ifndef _VPLIB_H_vpFree
#error _VPLIB_H_vpFree must be defined before including <vplib.h>
#endif

/**
 * vpList array hols vp identifiers
 * highnib of identifier is the entry number from the vp initialization array
 * lownib of identifier is the vp type
 * vpList index number specifies rambank: 0 ->first vp bank, _VPDEF_H_VPSLOTS-1 -> last vp bank
*/

char vpList[_VPDEF_H_VPSLOTS] @_VPLIB_H_vpList;
char vpFree @_VPLIB_H_vpFree;

/**
 * Initialize VP environment
 */
void vpInit() {
  char i;
  for (i=0; i<_VPDEF_H_VPSLOTS; i++) {
    vpList[i] = 0;
  }
}

char vpFind(long vpdef) {
  char i;
  char value = (vpdef-VPinitialize);
  value /= 17;
  value |= RomChar(vpdef);
  for (i=0; i<_VPDEF_H_VPSLOTS; i++) {
    if (vpList[i] == value) return i;
  }
  return _VPDEF_H_VPSLOTS;
}

/**
 * Install virtual peripheral
 *
 * @param vpdef Index from vpInit
 * @return Address of allocated vpBank (success) or 0 (failed)
 */
void vpInstall(long vpdef) {
  char i,type,vpram,vp;
  type = RomChar(vpdef);
  vpdef++;
  vpram = &vpData; //vpBank in highnib, vpType in lownib
  vp = (vpdef-VPinitialize);
  vp /= 17;
  vp <<= 4;
  vp |= type;
  for (i=0; i<_VPDEF_H_VPSLOTS; i++) {
    if (vpList[i] == 0) {
	  _SystemTemp = vp;
      vpList[i] = _SystemTemp;
	  vp = vpram & 0xF0;
      RomCopy(vp,vpdef,16);
      vpFree--;
	  return;
    }
#if defined _CHIP_SX18_ || defined _CHIP_SX20_ || defined _CHIP_SX28_
	vpram += 0x20;
#endif
#if defined _CHIP_SX48_ || defined _CHIP_SX52_
	vpram += 0x10;
#endif
  }
}

/**
 * Uninstall virtual peripheral
 *
 * @param vp Address of vpBank for currently installed virtual peripheral
 */
void vpUninstall(long vpdef) {
  char i = vpFind(vpdef);
  if (i < _VPDEF_H_VPSLOTS) {
    vpList[i] = 0;
    vpFree++;
  }
}

#pragma library 0
#endif
// End of file vplib.h
